/*
 *  This file is part of the FREGENE package.
 *  Copyright (c) Clive J Hoggart   <c.hoggart@imperial.ac.uk>
 *                Taane G Clark     <tgc@well.ox.ac.uk>
 *                Marc Chadeau      <m.chadeau@imperial.ac.uk>
 *
 *  FREGENE is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License, version 2, as
 *  published by the Free Software Foundation.
 *
 *  FREGENE is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */ 
//******************************************************************
//*
//*        REVISION HISTORY:
//*        $Log: biology.cc,v $
//*        Revision 1.11  2006/12/21 11:13:17  choggart
//*        some changes
//*
//*        Revision 1.10  2006/11/16 16:27:36  choggart
//*        loads of changes
//*
//*        Revision 1.9  2006/01/06 15:23:06  choggart
//*        Long time, many changes.
//*
//*        Revision 1.8  2005/10/04 13:07:04  choggart
//*        changes
//*
//*        Revision 1.7  2005/06/02 14:57:35  choggart
//*        Reverting back to gsl random number generation.
//*
//*        Revision 1.6  2005/05/24 11:19:40  choggart
//*        removed gsl random number generation to return ricardo's stuff
//*
//*        Revision 1.5  2005/05/19 13:50:58  choggart
//*        Moved rand to fregene directory. Removed bug reading in from xml.
//*
//*        Revision 1.4  2005/05/18 16:17:03  choggart
//*        Loads of changes - should commit more frequently
//*
//*        Revision 1.3  2005/03/23 15:14:32  choggart
//*        edits for hotspot model
//*
//*        Revision 1.2  2005/03/07 15:15:12  choggart
//*        removed windows formatting
//*
//*        Revision 1.1.1.1  2005/03/04 15:13:36  ejo31
//*        imported sources
//*
//*

#include "fregene.h"

using namespace std;

//******************************************************************
//* EXTERNALs parameters 
extern uint32_t MaxNbMutChromo;
extern uint32_t ChromoLength;
extern uint32_t StartOfInversion;
extern uint32_t EndOfInversion;
extern float StopFreqOfInv;
extern double  MutationRate; 
extern double MigrationRate;
extern uint32_t  NoGroups;
extern vector< vector<double> > migration;

//******************************************************************
//* Functions
//******************************************************************
void APPLY_MUTATION( uint32_t*, uint32_t*, const double,
                     uint32_t, const uint32_t, const uint32_t );

//bool ApplyTransmission(Recombinations &recom,
bool ApplyTransmission(Recomb2 &recom,
                       const uint32_t *_POPN, const uint32_t *_INDEX,
                       uint32_t *_NEWPOPN, uint32_t *_INDEXNEW,
                       uint32_t *Group_start, uint32_t *Group_size,
                       uint32_t *new_Group_start, uint32_t *new_Group_size,
                       uint32_t _PopnSizeNEWPOPN,
                       uint32_t _NoColPOPN, uint32_t _LastOccupiedElement,
		       bitset<size> &InvA,
		       bitset<size> &InvB,
		       vector<uint32_t> &InvertedSites )
{
   bool NewInversion = false;
   double Mut_x_Chromo = ChromoLength * MutationRate;
   vector<uint32_t> BreakPointVector;
   uint32_t i=0, k=0, gruppo=0, newGroup;
   uint32_t Chromo1 = 10000, Chromo2 = 10000, NewChromo=0;
   MaxNbMutChromo =0; // RE-CALCULATE MaxNbMutChromo;
   if (NoGroups==1) MigrationRate=0; //means no migration is possible

// FOR EACH GROUP here is the loop   
   for (gruppo=1;gruppo<NoGroups+1;gruppo++)
   {
      uint32_t Group_end = Group_start[gruppo] + Group_size[gruppo];
// WITHIN EACH GROUP here is the loop for each vector
      for( NewChromo = new_Group_start[gruppo];
           NewChromo < new_Group_start[gruppo] + new_Group_size[gruppo];
	   NewChromo++ ){         

// **********  SELECT two chromosomes (from the same island)********
           uint32_t new_Group_end;
           uint32_t TempPosition=0;
	   if( migration.size() > 0 ){
	     newGroup = SampleFromDiscrete( migration[gruppo-1] ) + 1;
	     new_Group_end = Group_start[ newGroup ] + Group_size[ newGroup ];
	     Chromo1 = SampleFromRange( Group_start[ newGroup ], new_Group_end );
	     Chromo2 = SampleFromRange( Group_start[ newGroup ], new_Group_end );
	   }
	   else{
	     if( myrand() > MigrationRate ){ //always true if MigrationRate=0
	       Chromo1 = SampleFromRange( Group_start[ gruppo ], Group_end );
	       Chromo2 = SampleFromRange( Group_start[ gruppo ], Group_end );
	     }
	     else{
	       newGroup = SampleFromRange( 1, NoGroups+1 );
	       new_Group_end = Group_start[ newGroup ] + Group_size[ newGroup ];
	       Chromo1 = SampleFromRange( Group_start[ newGroup ], new_Group_end );
	       Chromo2 = SampleFromRange( Group_start[ newGroup ], new_Group_end );
	     }
	   }

	   //Start of Inversion code
	   bool RecombinationInInversion;
	   if( EndOfInversion > 0 ){
	     if( (InvA.none() && NewChromo == 0) || StopFreqOfInv == 0 ){
	       do{
		 RecombinationInInversion = false;
		 BreakPointVector.clear();
		 recom.GetRecombinations(BreakPointVector);
		 for (i=0;i<BreakPointVector.size();i++){
		   if( StartOfInversion < BreakPointVector[i] &&
		       BreakPointVector[i] < EndOfInversion ){
		     RecombinationInInversion = true;
		   }
		 }
	       }while( RecombinationInInversion );
	       if( StopFreqOfInv != 0 ){
		 InvB.set(0);
		 NewInversion = true;
		 InvertedSites.clear();
	       }
	     }else{
	       int RecomsToLeftOfInversion;
	       do{
		 RecombinationInInversion = false;
		 RecomsToLeftOfInversion = 0;
		 BreakPointVector.clear();
		 recom.GetRecombinations(BreakPointVector);
		 for(i=1;i<BreakPointVector.size();i++){
		   if( BreakPointVector[i] < StartOfInversion )
		     RecomsToLeftOfInversion++;
		   if( StartOfInversion < BreakPointVector[i] && BreakPointVector[i] < EndOfInversion ){
		     RecombinationInInversion = true;
		   }
		 }
	       }while( RecombinationInInversion && (InvA.test(Chromo1) != InvA.test(Chromo2)) );
	       if( RecomsToLeftOfInversion%2 == 0 )
		 InvB[NewChromo] = InvA[Chromo1];
	       else
		 InvB[NewChromo] = InvA[Chromo2];
	     }
	   }// End of Inversion code
	   else{
	     BreakPointVector.clear();
	     recom.GetRecombinations(BreakPointVector);
	   }
	   
//******************************
// UN-RECOMBINANT CHROMOSOMEs **
//******************************
	   if( BreakPointVector.size() == 0 ){
	     // No recombination, therefore chromo is transmitted entirely
	     // Choose one Chromosome and work out the old mutations and the new to add
	     // the function performs the migration and will perform also selection
	     // Transfer Chromosome: Old=UnrecombinantChromo, new=NewChromo;
	     for (k=0; k<_NoColPOPN;k++){
	       TempPosition = _POPN[ _INDEX[Chromo1]+k ];
	       if( NewInversion && NewChromo==0 &&
		   StartOfInversion < TempPosition && TempPosition < EndOfInversion ){
		 TempPosition = StartOfInversion + EndOfInversion - TempPosition;
		 InvertedSites.push_back(TempPosition);
	       }
	       _NEWPOPN[ _INDEXNEW[NewChromo]+ k ] = TempPosition;
	       if (_NEWPOPN[ _INDEXNEW[NewChromo]+ k ]==0)
		 break;
	     }
	     // k is the first available position
	     APPLY_MUTATION( _NEWPOPN, _INDEXNEW, Mut_x_Chromo, k, NewChromo, _PopnSizeNEWPOPN );
	   }else{
// *** END OF non-recombinant chromosome transmission
           
//*****************************************************************************
// RECOMBINANT CHROMOSOMEs *******
//*****************************************************************************

           uint32_t position=0;
           uint32_t column=0;

// ***************** COPY FIRST CHROMO ********************
           for (k=0; k<_NoColPOPN;k++){
              if (_POPN[ _INDEX[Chromo1]+k ]==0)
		break;
              else
		TempPosition=_POPN[ _INDEX[Chromo1]+k ];
	      if( NewInversion && NewChromo==0 &&
		  StartOfInversion < TempPosition && TempPosition < EndOfInversion ){
		TempPosition = StartOfInversion + EndOfInversion - TempPosition;
	      }
	      //for first chromosome we start from first region 
              for (i=0;i<BreakPointVector.size();i++){
                 if( BreakPointVector[i]<TempPosition && TempPosition<=BreakPointVector[i+1]){
                    _NEWPOPN[ _INDEXNEW[NewChromo] + position ] = TempPosition;
		    if( NewInversion && NewChromo==0 &&
			StartOfInversion < TempPosition && TempPosition < EndOfInversion )
		      InvertedSites.push_back(TempPosition);
                    position++;
                    break;
                 }
                 i++;
// if value falls in this other region then it should not be transmitted
                 if (BreakPointVector[i]<TempPosition && TempPosition<=BreakPointVector[i+1]) break;
//EVENTUALLY THE LOOP WILL BREAK SINCE THE VALUE NECESSARELY FALL IN ONE OF THE REGIONS
              }
           }
           //in case no mutations is incorporated from second chromo
           _NEWPOPN[ _INDEXNEW[NewChromo]+ position ]=0;
           
           //***************** COPY SECOND CHROMO*****************
           column=0;
           for (k=0; k<_NoColPOPN;k++){
              if (_POPN[ _INDEX[Chromo2]+k ]==0)
		break;
              else
		TempPosition=_POPN[ _INDEX[Chromo2]+k ];
              if( NewInversion && NewChromo==0 &&
		  StartOfInversion < TempPosition && TempPosition < EndOfInversion ){
                TempPosition = StartOfInversion + EndOfInversion - TempPosition;
	      }
              //for second chromosome we start from second region 
              for( i=1; i < BreakPointVector.size(); i++ ){
                 if (BreakPointVector[i]<TempPosition && TempPosition<=BreakPointVector[i+1]){
                    _NEWPOPN[ _INDEXNEW[NewChromo] + position ] = TempPosition;
		    if( NewInversion && NewChromo==0 &&
			StartOfInversion < TempPosition && TempPosition < EndOfInversion )
		      InvertedSites.push_back(TempPosition);
		    position++;
                    break;
                 }
                 i++;
	// if value falls in this other region then it should not be transmitted
                 if (BreakPointVector[i]<TempPosition && TempPosition<=BreakPointVector[i+1]) break;
 //EVENTUALLY THE LOOP WILL BREAK SINCE THE VALUE NECESSARELY FALL IN ONE OF THE REGIONS
              }
           }
           _NEWPOPN[ _INDEXNEW[NewChromo]+ position ]=0;
           k=position;	
	   
           APPLY_MUTATION( _NEWPOPN, _INDEXNEW, Mut_x_Chromo, position,
			   NewChromo, _PopnSizeNEWPOPN );
	   }

// *** END OF recombinant chromosome transmission
         
      }// end of chromosome loop
   }// end of group loop

   _LastOccupiedElement=_INDEXNEW[NewChromo-1] + k;//of course occupied by a zero

   return NewInversion;
}

void APPLY_MUTATION( uint32_t *_POPN, uint32_t *_INDEX,
		     const double Mut_x_Chromo,
                     uint32_t k, const uint32_t Chromo,
		     const uint32_t _PopnSizePOPN )
{
   uint32_t i;
   uint32_t nbMutToApply = genpoi( Mut_x_Chromo );
   for( i = 0; i < nbMutToApply; i++ ){
      uint32_t baseChosen;
      baseChosen=SampleFromRange(1,ChromoLength);
      // apply mutation on the same chromosome that we are working on
      _POPN[ _INDEX[ Chromo ]+k ]=baseChosen;
      k++;
   }            
   // put a zero after the new mutation
   _POPN[ _INDEX[Chromo] + k]=0;
//***     END OF MUTATION        *****
   // update max nub of mutation
   if( MaxNbMutChromo < k+1 ) MaxNbMutChromo = k+1;
   // define position for next chromosome
   if( Chromo != _PopnSizePOPN-1 )
      _INDEX[Chromo+1] = _INDEX[Chromo] + k + 1;
   // we do not need position of n+1 chromo, we've only n chromos            
}
