/*
 *  This file is part of the FREGENE package.
 *  Copyright (c) Clive J Hoggart   <c.hoggart@imperial.ac.uk>
 *                Taane G Clark     <tgc@well.ox.ac.uk>
 *                Marc Chadeau      <m.chadeau@imperial.ac.uk>
 *
 *  FREGENE is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License, version 2, as
 *  published by the Free Software Foundation.
 *
 *  FREGENE is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */ 
#include <string.h>

#ifndef MA_STRUCT_H

#define MA_STRUCT_H


////////////////////////////////////////////////
////---> Range structure

typedef struct
{
	long start;
	long end;
} RANGE;

////////////////////////////////////////////////
////---> Range structure unsigned

typedef struct
{
	unsigned long start;
	unsigned long end;
} URANGE;

////////////////////////////////////////////////
////---> Peak Picking Parameter struct

class PEAK_PICKING_PARAM
{
public:
	char				manip[1024];
	char				image[1024];
	char				author[256];
	int				dimx;
	int				dimy;
	short				min_nb_pix;
	short				max_nb_pix;
	short				min_peak_signal;
	short				baseline_parameter;
	short				stop_parameter;
	double			pcr;
	int				use_filter;
	int				filter;
	int				fpar1;
	double			fpar2;
	RANGE          xr, yr;
	int				nb_peaks;
	bool				fluo, halogen;
	RANGE				dynamic;
	double			ExposureTime;
	double			LightIntensity;


public:

	void				set_image_name(char *s){strcpy(image,s);}
	void				set_image_author(char *s){strcpy(author,s);}
	void				set_image_size(int dx, int dy){dimx=dx; dimy=dy;}
	void				set_image_zone(RANGE  x, RANGE y) {xr=x; yr=y;}
	void				set_ppp_image(char *s, char *a, int dx, int dy, RANGE  x, RANGE y)
						{
							set_image_name(s);
							set_image_author(a);
							set_image_size(dx, dy);
							set_image_zone(x, y);
						}

	void				set_min_nb_pix(short mnp1){min_nb_pix=mnp1;}
	void				set_max_nb_pix(short mnp2){max_nb_pix=mnp2;}
	void				set_min_peak_signal(short mps){min_peak_signal=mps;}
	void				set_bp(short bp){baseline_parameter=bp;}
	void				set_sp(short sp){stop_parameter=sp;}
	void				set_pcr(double p){pcr=p;}
	void				set_use_filter(int ufil){use_filter=ufil;}
	void				set_filter(int fil){filter=fil;}
	void				set_fpar1(int par){fpar1=par;}
	void				set_fpar2(double par){fpar2=par;}
	void				set_ppp_detection(short mnp1,short mnp2, short mps, short bp, short sp, double p, int ufil, int fil, int par1, double par2)
						{
							set_min_nb_pix(mnp1);
							set_max_nb_pix(mnp2);
							set_min_peak_signal(mps);
							set_bp(bp);
							set_sp(sp);
							set_pcr(p);
							set_use_filter(ufil);
							set_filter(fil);
							set_fpar1(par1);
							set_fpar2(par2);
						}

	void				set_manip_file(char *s){strcpy(manip,s);}
	void				set_dynamic(RANGE dyn){dynamic=dyn;}
	void				set_exposure_time(double ET){ExposureTime=ET;}
	void				set_light_intensity(double LI){LightIntensity=LI;}
	void				set_fluo_halo(bool fl, bool halo){fluo=fl;halogen=halo;}
	void				set_ppp_acquisition(char *s, RANGE dyn, double ET, double LI, bool fl, bool halo)
						{
							set_manip_file(s);
							set_dynamic( dyn);
							set_exposure_time( ET);
							set_light_intensity( LI);
							set_fluo_halo( fl,  halo);
						}

	void				set_nbpeaks(int nbp){nb_peaks=nbp;}


	int set_ppp(const char *s1, const char *s2, const char *s3, int dx, int dy, short mnp1, short mnp2, short mps, short bp, short sp, double p, bool fil, RANGE x, RANGE y, int np,
									bool fl, bool halo, RANGE dyn, double ET, double LI)
						{
							strcpy(manip,s1);
							strcpy(image,s2);
							strcpy(author,s3);
							dimx=dx;
							dimy=dy;
							filter=fil;
							min_nb_pix=mnp1;
							max_nb_pix=mnp2;
							min_peak_signal=mps;
							baseline_parameter=bp;
							stop_parameter=sp;
							pcr=p;
							memcpy(&xr,&x,sizeof(RANGE));
							memcpy(&yr,&y,sizeof(RANGE));
							nb_peaks=np;
							fluo=fl;
							halogen=halo;
							dynamic=dyn;
							ExposureTime=ET;
							LightIntensity=LI;

							return 0;
						}
	int				init_ppp() {RANGE r={0,0}; set_ppp("","","",0,0,0,0,0,0,0,0,0,r,r,0,0,0,r,0,0); return 0;}


};


////////////////////////////////////////////////
////---> Peak Matching Parameter struct

class PEAK_MATCHING_PARAM
{
public:
	char				manip[1024];
	char				**ppfile;
	unsigned short nb_file_treated;
	char				ref_file_name[1024];
	unsigned char  ref_file;
	unsigned short continuity;
	unsigned char  num_col;
	unsigned char  merge_dist;
	unsigned short max_translation;
	unsigned short crims;




public:

	int set_ppm(const char *s1, char *s[], unsigned char p0, const char *s2, unsigned char p1, unsigned short p2, unsigned char p3,
									unsigned char p4, unsigned short p5, unsigned short p6)
						{
							strcpy(manip,s1);
							//strcpy(image,s2);
							s=NULL;
							nb_file_treated =p0;
							strcpy(ref_file_name,s2);
							ref_file=p1;
							continuity=p2;
							num_col=p3;
							merge_dist=p4;
							max_translation=p5;
							crims=p6;
							return 0;
						}
	int init_ppm() {set_ppm("",NULL,0,"",0,0,0,0,0,0); return 0;}


};



////////////////////////////////////////////////
////---> Integer Point

typedef struct
{
	int x;
	int y;
} IPOINT;


////////////////////////////////////////////////
////---> float   Point

typedef struct
{
	double x;
	double y;
} FPOINT;


////////////////////////////////////////////////
////---> Stat Variables
typedef struct
{
	RANGE		min_max;
	int		size;
	double	mean;
	double	sigma;
} STATS;


////////////////////////////////////////////////
////---> Zone struct
typedef struct
{
	IPOINT upper_left;
	IPOINT lower_right;
} ZONE;


////////////////////////////////////////////////
////---> Program information (PROG_INFO) struct
typedef struct
{
	char *sSource;
	char *sVersion;
} PROG_INFO;


////////////////////////////////////////////////
////---> Creation/modification information (CREA_MODIF_INFO) struct
typedef struct
{
	char sDate[9];
	char *sAuthor;
	PROG_INFO ProgInfo;
} CREA_MODIF_INFO;


////////////////////////////////////////////////
////---> Single Nucleotide Polymorphism (SNP) struct
typedef struct
{
	unsigned long iRsNb;
	int iNbTimes;
	unsigned long *iPos;
	unsigned char cCode;
	unsigned char iClass;
} SNP;


////////////////////////////////////////////////
////---> Contig info struct
typedef struct
{
	char *sName;
	unsigned long iPos;
} CTG_INFO;


////////////////////////////////////////////////
////---> Dna Index File info struct
typedef struct
{
	char *sFileRelPath;
	unsigned short iStart;
	unsigned short iEnd;
} IDX_FILE_INFO;


////////////////////////////////////////////////
////---> Histogram bucket struct
typedef struct
{
	RANGE Range;
	unsigned long iValue;
} H_BUCKET;

#endif

