/*
 *  This file is part of the FREGENE package.
 *  Copyright (c) Clive J Hoggart   <c.hoggart@imperial.ac.uk>
 *                Taane G Clark     <tgc@well.ox.ac.uk>
 *                Marc Chadeau      <m.chadeau@imperial.ac.uk>
 *
 *  FREGENE is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License, version 2, as
 *  published by the Free Software Foundation.
 *
 *  FREGENE is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */ 
#include <cstring>
#include <cstdio>
#include <cstdlib>
#include "maerrno.h"

#ifndef MAEXCEPCLS_H

#define MAEXCEPCLS_H


/* Exceptions Class*/

class MaException  //Base Class of ma exceptions
{
      public:
	virtual void GetErrorMessage () = 0;
};


/* File Error Exceptions */
class MaFileOpenException : public MaException
{
	char *str;

	public :	
		MaFileOpenException(char *fname);
		virtual ~MaFileOpenException ();
		const char *GetFileName ();

		virtual void GetErrorMessage ();

};


/* Error of No Read in Command Line */
class MaCmdLineReadException : public MaException
{
	char *str;

	public :	
		MaCmdLineReadException(char *option);
		virtual ~MaCmdLineReadException();

		virtual void GetErrorMessage ();
};



/* Error of No Read in Command Line */
class MaXmlFileException : public MaException
{
	char *str;
	int  Error;

	public :	
		MaXmlFileException(int e, const char *mess="");
		virtual ~MaXmlFileException();

		virtual void GetErrorMessage ();
		void SetError(int n=MA_XML_FILE_ERROR) {Error=n;}
		void SetMessage(const char *mess) {strcpy(str,mess);}
};


#endif
