/*
 *  This file is part of the FREGENE package.
 *  Copyright (c) Clive J Hoggart   <c.hoggart@imperial.ac.uk>
 *                Taane G Clark     <tgc@well.ox.ac.uk>
 *                Marc Chadeau      <m.chadeau@imperial.ac.uk>
 *
 *  FREGENE is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License, version 2, as
 *  published by the Free Software Foundation.
 *
 *  FREGENE is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */ 
#ifndef MATHS_OUTILS_H

#define MATHS_OUTILS_H

#define IA 16807
#define IM 2147483647
#define AM (1.0/IM)
#define IQ 127773
#define IR 2836
#define NTAB 32
#define NDIV (1+(IM-1)/NTAB)
#define EPS 1.2e-7
#define RNMX (1.0-EPS)

#define max_line 1024
#define max_name 1024
#define max_int  65534
#define max_nb   65534
#define pi       3.141592
#define D4       4
#define D8       8
#define D2       2

#ifndef SWAP
#define SWAP(a,b) {swap=(a); (a)=(b); (b)=swap;}
#endif

#ifndef max
#define max(a,b)            (((a) > (b)) ? (a) : (b))
#endif

#ifndef min
#define min(a,b)            (((a) < (b)) ? (a) : (b))
#endif

#ifndef LOG2
#define LOG2(a)            (log(a)/log(2))
#endif


double disteuc(int x1, int y1, int x2, int y2);
double dist(int d, int x1, int y1, int x2, int y2);
void   tri1(double *p, int *ip, int n);
int    NbpToRadius(int & nbp, int d);
long	fact(int n);
double	NbpToSigma(int n, double y0, double a, double b);
double	ran1(long *idum);
void tri(unsigned short *u1,int N);
void tri_inv(unsigned short *u1,int N);
void D_tri_inv(double *u1,int N);
unsigned short select(unsigned long k, unsigned long n, unsigned short *arr);
void hpsort(unsigned long n, float ra[]);
void shell(unsigned long n, unsigned short a[]);
long lselect(unsigned long k, unsigned long n, unsigned long *arr);

#endif
