/*
 *  This file is part of FREGENE program.
 *  Copyright (c) Clive J Hoggart   <c.hoggart@imperial.ac.uk>
 *                Taane G Clark     <tgc@well.ox.ac.uk>
 *                Marc Chadeau      <m.chadeau@imperial.ac.uk>
 *
 *  FREGENE is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License, version 2, as
 *  published by the Free Software Foundation.
 *
 *  FREGENE is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */ 


#include <math.h>
#include <iostream>
#include <fstream>
#include <iomanip>
#include <vector>
#include <bitset>
#include "rand.h"

using namespace std;
const unsigned long size = 1000000;
// **********************************************
// *********   EXTERN ****************
// **********************************************
extern long RandomSeed; //for DefineDiseaseSNP(), 

double Baseline;
vector<double> JointRR;

extern vector<unsigned long> DiseaseSNP; //for DefineDiseaseSNP()
extern double *RR; //for DefineDiseaseSNP()
extern double *DiseaseFreq1; //for DefineDiseaseSNP()
extern double *DiseaseFreq2; //for DefineDiseaseSNP()
extern ofstream logstream;
extern double sigma;

// ******************************************************************
// **   GetDiseaseSNP: FIND CAUSATIVE SNPs	       **************
// **   this function give values to the global vars: DiseaseSNP1,
// **   DiseaseSNP2, ...
// ******************************************************************
// return a value btw 0 and _SNPsize - 1 for DiseaseSNP1, so then
// need to be increased by one // .. to get the right causative SNP
void GetDiseaseSNP(unsigned long distance, short _disease_snps,
                   unsigned long _SNPsize, double *_PopnFreq )
{
  long a;
  unsigned long b;
  if( distance != _SNPsize ){
     for( int i = 0; i < _disease_snps/2; i++ ){
        do{
           DiseaseSNP[2*i] = SampleFromRange(0,_SNPsize);
        }while( _PopnFreq[ DiseaseSNP[2*i] ] < DiseaseFreq1[2*i] ||
                _PopnFreq[ DiseaseSNP[2*i] ] > DiseaseFreq2[2*i] );
	a = DiseaseSNP[2*i] - distance;
        if( a < 0 )
           a = 0;
        b = DiseaseSNP[2*i] + distance;
        if( b > _SNPsize )
           b = _SNPsize;
        do{
           DiseaseSNP[2*i+1] = SampleFromRange(a,b);
        }while( _PopnFreq[ DiseaseSNP[2*i+1] ] < DiseaseFreq1[2*i+1] ||
                _PopnFreq[ DiseaseSNP[2*i+1] ] > DiseaseFreq2[2*i+1] );
     }
  }
  else{
     for( int i = 0; i < _disease_snps; i++ ){
       bool duplicate;
       do{
	 duplicate = false;
	 DiseaseSNP[i] = SampleFromRange(0,_SNPsize);
	 for( int ii = 0; ii < i; ii++ )
	   if( DiseaseSNP[i] == DiseaseSNP[ii] )
	     duplicate = true; 
       }while( _PopnFreq[ DiseaseSNP[i] ] < DiseaseFreq1[i] ||
	       _PopnFreq[ DiseaseSNP[i] ] > DiseaseFreq2[i] || duplicate );
     }
  }
}

void SetDiseaseModel_quant( unsigned short DiseaseLoci,
			    double *_PopnFreq )
{
   double pa;
   for( int i = 0; i < DiseaseLoci; i++ ){
      pa = _PopnFreq[ DiseaseSNP[i] ];
      RR[i] = sqrt( sigma*sigma*RR[i]*RR[i] / (2.0*pa*(1-pa)*(1-RR[i]*RR[i]) ) );
   }
}

void SetDiseaseModel( short _model, double prevalence,
		      unsigned short DiseaseLoci,
                      double *_PopnFreq, vector<double>* h )
{
   Baseline = prevalence;
   double pa = 0.0, rr;
   vector<double> theta;

   JointRR.clear();

   if( _model != 0 ){
      for( int i = 0; i < DiseaseLoci; i++ ){
         if( _model == 3 ){// Interaction model
//          Baseline = prevalence / ( h[3]*h[3] * RR[0]*RR[0]*RR[0]*RR[0]
//                                    + 2.0*(h[3]*h[2]+h[3]*h[1]) * RR[0]*RR[0]
//                                    + 2.0*(h[0]*h[3]+h[1]*h[2]) * RR[0]
//                                    + h[1]*h[1] + h[2]*h[2] + 2.0*h[0]*h[1] + 2.0*h[0]*h[2] + h[0]*h[0] );
//             pa = _PopnFreq[ DiseaseSNP[2*i] ];
//             pb = _PopnFreq[ DiseaseSNP[2*i+1] ];
//             theta.push_back( (pa*pa*RR[2*i]*RR[2*i] + 2.0*pa*(1.0-pa)*RR[2*i] + (1.0-pa)*(1.0-pa))
//                              *(pb*pb*RR[2*i+1]*RR[2*i+1] + 2.0*pb*(1.0-pb)*RR[2*i+1] + (1.0-pb)*(1.0-pb)));

            double r1 = RR[2*i];
            double r2 = RR[2*i+1];
            theta.push_back(   h[i][0]*h[i][0]              + h[i][1]*h[i][1]*r2*r2
                             + h[i][2]*h[i][2]*r1*r1        + h[i][3]*h[i][3]*r1*r1*r2*r2
                             + 2.0*h[i][0]*h[i][1]*r2       + 2.0*h[i][0]*h[i][2]*r1
                             + 2.0*h[i][1]*h[i][2]*r1*r2    + 2.0*h[i][0]*h[i][3]*r1*r2
                             + 2.0*h[i][3]*h[i][2]*r1*r1*r2 + 2.0*h[i][3]*h[i][1]*r1*r2*r2 );

            if( h[i][3] != 0.0 ){
               double alpha = 2.0*(h[i][1]*h[i][3]+h[i][2]*h[i][3]) / (h[i][3]*h[i][3]);
               double beta = 2.0*(h[i][0]*h[i][3]+h[i][1]*h[i][2]) / (h[i][3]*h[i][3]);
               double gamma = ( h[i][0]*h[i][0] + h[i][1]*h[i][1] + h[i][2]*h[i][2] 
                                + 2*h[i][0]*(h[i][1]+h[i][2]) - theta[i] ) / (h[i][3]*h[i][3]);
               double P = -alpha*alpha/12 -gamma;
               double Q = -alpha*alpha*alpha/108 + alpha*gamma/3 - beta*beta/8;
               double R = Q/2 + sqrt(Q*Q/4 + P*P*P/27);
               double U = pow(R,1.0/3);
               double y = -5.0*alpha/6 - U + P/(3.0*U);
               double W = sqrt(alpha+2.0*y);
// The roots are, we take the forth one.
                cout << 0.5 * ( W - sqrt(-(3.0*alpha + 2.0*y + 2.0*beta/W )) ) << " "
                     << 0.5 * ( W + sqrt(-(3.0*alpha + 2.0*y + 2.0*beta/W )) ) << " "
                     << 0.5 * ( -W - sqrt(-(3.0*alpha + 2.0*y - 2.0*beta/W )) ) << " "
                     << 0.5 * ( -W + sqrt(-(3.0*alpha + 2.0*y - 2.0*beta/W )) ) << endl;
               rr = 0.5 * ( -W + sqrt(-(3.0*alpha + 2.0*y - 2.0*beta/W )) );
               JointRR.push_back( rr );
            }
            else{
               rr = (theta[i] - (h[i][1]*h[i][1] + h[i][2]*h[i][2] + h[i][0]*h[i][0]
                                 + 2.0*h[i][0]*h[i][1] + 2.0*h[i][0]*h[i][2])) / (2.0*h[i][1]*h[i][2]);
               JointRR.push_back( rr );
            }
            logstream << rr*rr*rr*rr*h[i][3]*h[i][3] + 2.0*rr*rr*(h[i][3]*h[i][2] + h[i][3]*h[i][1])
               + 2.0*rr*(h[i][0]*h[i][3] + h[i][1]*h[i][2])
               + h[i][1]*h[i][1] + h[i][2]*h[i][2] + h[i][0]*h[i][0]
               + 2.0*h[i][0]*h[i][1] + 2.0*h[i][0]*h[i][2] << endl;
            logstream << "Risk ratio for disease locus " << i << " " << JointRR[i] << endl;
         }
         else if( _model == 2 ){// AND model
//          Baseline = prevalence / ( h[3]*h[3] * RR[0]
//                                    + 2.0*(h[3]*h[2]+h[3]*h[1]) * RR[0]
//                                    + 2.0*(h[0]*h[3]+h[1]*h[2]) *RR[0]
//                                    + h[1]*h[1] + h[2]*h[2] + 2.0*h[0]*h[1] + 2.0*h[0]*h[2] + h[0]*h[0] );
            double r1 = RR[2*i];
            double r2 = RR[2*i+1];
            theta.push_back(   h[i][0]*h[i][0]              + h[i][1]*h[i][1]*r2*r2
                             + h[i][2]*h[i][2]*r1*r1        + h[i][3]*h[i][3]*r1*r1*r2*r2
                             + 2.0*h[i][0]*h[i][1]*r2       + 2.0*h[i][0]*h[i][2]*r1
                             + 2.0*h[i][1]*h[i][2]*r1*r2    + 2.0*h[i][0]*h[i][3]*r1*r2
                             + 2.0*h[i][3]*h[i][2]*r1*r1*r2 + 2.0*h[i][3]*h[i][1]*r1*r2*r2 );

//             pa = _PopnFreq[ DiseaseSNP[2*i] ];
//             pb = _PopnFreq[ DiseaseSNP[2*i+1] ];
//             theta.push_back( (pa*pa*RR[2*i]*RR[2*i] + 2.0*pa*(1.0-pa)*RR[2*i] + (1.0-pa)*(1.0-pa))
//                              *(pb*pb*RR[2*i+1]*RR[2*i+1] + 2.0*pb*(1.0-pb)*RR[2*i+1] + (1.0-pb)*(1.0-pb)));

            rr = (theta[i] - (h[i][1]*h[i][1] + h[i][2]*h[i][2] + h[i][0]*h[i][0]
                              + 2.0*h[i][0]*h[i][1] + 2.0*h[i][0]*h[i][2]) ) /
               (h[i][3]*h[i][3] + 2.0*(h[i][3]*h[i][2] + h[i][3]*h[i][1]
                                       + h[i][0]*h[i][3] + h[i][1]*h[i][2]));
            JointRR.push_back( rr );
            logstream << rr*h[i][3]*h[i][3] + 2.0*rr*(h[i][3]*h[i][2] + h[i][3]*h[i][1])
               + 2.0*rr*(h[i][0]*h[i][3] + h[i][1]*h[i][2])
               + h[i][1]*h[i][1] + h[i][2]*h[i][2] + h[i][0]*h[i][0]
               + 2.0*h[i][0]*h[i][1] + 2.0*h[i][0]*h[i][2] << endl;
            logstream << "Risk ratio for disease locus " << i << " " << JointRR[i] << endl;
         }
         else{
            pa = _PopnFreq[ DiseaseSNP[i] ];
            theta.push_back( (pa*pa*RR[i]*RR[i] + 2.0*pa*(1.0-pa)*RR[i] + (1.0-pa)*(1.0-pa)) );
         }
//         Baseline = prevalence / ( h[i][3]*h[3] * RR[0]*RR[0]*RR[0]*RR[0]
//                                    + 2.0*(h[3]*h[2]+h[3]*h[1]) * RR[0]*RR[0]
//                                    + 2.0*(h[0]*h[3]+h[1]*h[2]) * RR[0]
//                                    + h[1]*h[1] * RR[0]*RR[0]
//                                    + h[2]*h[2] * RR[0]*RR[0]
//                                    + 2.0*h[0]*h[1] * RR[0]
//                                    + 2.0*h[0]*h[2] * RR[0]
//                                    + h[0]*h[0] );
//      }
         Baseline /= theta[i];
         logstream << "Heritability: " << theta[i] << endl;
      }
   }
   logstream << "Baseline: " << Baseline << endl;
}

// ******************************************************************
// **  GetPhenotype: generate phenotype:	       **************
// **  this function give values to the global vars: DiseaseSNP1,
// **  DiseaseSNP2, ...
// ******************************************************************

double GetPhenotype_quant(unsigned short DiseaseLoci, unsigned long _Chromo1,
                          unsigned long _Chromo2, vector<bitset<size> > &_POPN )
{
   double y=0;
   for( int i = 0; i < DiseaseLoci; i++ ){
      y += (_POPN[_Chromo1][ DiseaseSNP[i] ] + _POPN[_Chromo2][ DiseaseSNP[i] ] - 1) * RR[i];
   }
   y += gennor( 0, sigma );
   return y;
}

short GetPhenotype(short _model, unsigned short DiseaseLoci, unsigned long _Chromo1,
                   unsigned long _Chromo2, vector<bitset<size> > &_POPN )
{
  short X1=0, X2=0, status=0;
  double ProbAffected = Baseline;
  for( int i = 0; i < DiseaseLoci; i++ ){
     if( _model == 3 ){//Interaction model
        X1 = _POPN[_Chromo1][ DiseaseSNP[2*i] ] + _POPN[_Chromo2][ DiseaseSNP[2*i] ];
        X2 = _POPN[_Chromo1][ DiseaseSNP[2*i+1] ] + _POPN[_Chromo2][ DiseaseSNP[2*i+1] ];
        ProbAffected *= pow( JointRR[i], X1*X2 );
     }
     else if( _model == 2 ){//Joint AND model
        X1 = _POPN[_Chromo1][ DiseaseSNP[2*i] ] + _POPN[_Chromo2][ DiseaseSNP[2*i] ];
        X2 = _POPN[_Chromo1][ DiseaseSNP[2*i+1] ] + _POPN[_Chromo2][ DiseaseSNP[2*i+1] ];
        if( X1 > 0 && X2 > 0 ){
           ProbAffected *= JointRR[i];
        }
     }
     else if( _model == 1 ){//Additive model
        X1 = _POPN[_Chromo1][ DiseaseSNP[i] ] + _POPN[_Chromo2][ DiseaseSNP[i] ];
        ProbAffected *= pow( RR[i], X1 );
     }
     else
        ProbAffected = 0.5;
  }
  if( ProbAffected > 1 )
     cout << "Probablilty of affected = " << ProbAffected << endl;;
  if( ProbAffected > myrand() )
     status=1;
  return status;
}
