/*
 *  This file is part of the FREGENE package.
 *  Copyright (c) Clive J Hoggart   <c.hoggart@imperial.ac.uk>
 *                Marc Chadeau      <m.chadeau@imperial.ac.uk>
 *
 *  FREGENE is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License, version 2, as
 *  published by the Free Software Foundation.
 *
 *  FREGENE is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */ 
#include <iostream>
#include <fstream>
#include <cstdio>
#include <cstdlib>
#include <cmath>
#include <cstring>
#include <iostream>
#include <vector>
#include <memory>
#include <algorithm>

using namespace std;

#define SUCCESS_EXEC		1
#define SEPS "/"
#define CEPS '/'

void WriteCounts( uint32_t *POPN_NEW, uint32_t *INDEX_NEW, uint32_t row,
                  uint32_t ChromoLength, char* filename )
{
  uint32_t i, j=0;
  vector<uint32_t> MEGAVECTOR(ChromoLength+1, 0 );
  for(i=0; i<row; i++){
    j=0;
    while( POPN_NEW[ INDEX_NEW[i]+j ] != 0 ){
      MEGAVECTOR[ POPN_NEW[ INDEX_NEW[i]+j ] ]++;
      j++;
    }
  }
  ofstream outstream;
  outstream.open(filename);
  outstream << "Position  MAF" << endl;
  for( j=0; j<ChromoLength+1; j++ ){
    if( MEGAVECTOR[j] != 0 ){
      outstream << j << " " << (float)MEGAVECTOR[j]/row << endl; 
    }
  }
  outstream.close();
}

// ******************************************************
// *** HOMOZIGOSITY: to work out homozigosity and *******
// *** swap derived alleles at freq > 50% ***************
// ******************************************************
double homozigosity( uint32_t *POPN, uint32_t *INDEX,
                     uint32_t *POPN_NEW, uint32_t *INDEX_NEW,
                     uint32_t row, uint32_t ChromoLength,
                     bool flag, uint32_t *NoColPOPN, char* filename )
{
  double hetrozigosity=0.0;

  uint32_t i, j=0, k=0, kk=0, polymorphisms=0;
  vector<uint32_t> MEGAVECTOR(ChromoLength+1, 0 );

   for(i=0; i<row; i++){
      j=0;
      while( POPN[ INDEX[i]+j ] != 0 ){
         MEGAVECTOR[ POPN[ INDEX[i]+j ] ]++;
         j++;
      }
   }

   for( j=0; j<ChromoLength+1; j++ ){
      if( MEGAVECTOR[j] != 0 ){
         polymorphisms++;
         double freq = (double)MEGAVECTOR[j] / row;
         hetrozigosity += 2.0 * ( 1.0 - freq ) * freq;
      }
   }
   hetrozigosity /= ChromoLength;

   vector< uint32_t > pointer( polymorphisms, 0 );
   vector< uint32_t > vec_temp;

   for( j = 0; j < ChromoLength+1; j++ ){
     if( MEGAVECTOR[j] != 0 ){
       pointer[k] = j;
       k++;
     }
   }
   INDEX_NEW[0] = INDEX[0];
   for( i = 0; i < row; i++ ){
     vec_temp.clear();
     for( k = 0; k < *NoColPOPN; k++ ){
       if( POPN[ INDEX[i] + k ] == 0 ) break;
       vec_temp.push_back( POPN[ INDEX[i] + k ] );
     }
     if( flag ){
       for( j = 0;  j < polymorphisms; j++ ){
	 if( MEGAVECTOR[ pointer[j] ] > row / 2 ){
	   vec_temp.push_back( pointer[j] );
	   k++;
	 }
       }
     }
     if( k != 0 )
       sort( vec_temp.begin(), vec_temp.end() );
     kk=0;
     j=1;
     while( j < k+1 ){
       if( vec_temp[j] != vec_temp[j-1] ){
	 POPN_NEW[ INDEX_NEW[i] + kk ] = vec_temp[j-1];
	 kk++;
	 j++;
       }
       else
	 j+=2;
     }
     POPN_NEW[ INDEX_NEW[i] + kk ] = 0;
     kk++;
     if( i+1 < row )
       INDEX_NEW[i+1] = INDEX_NEW[i] + kk;
     if( kk > *NoColPOPN )
       *NoColPOPN=kk;
   }

   if( *filename )
      WriteCounts( POPN_NEW, INDEX_NEW, row, ChromoLength, filename );

   return 1-hetrozigosity;
}

// ******************************************************
///* GetFilePath: return the pathname
// ******************************************************

int GetFPath(char *fname, char *pathname)
{
   char *token;
   int tt=strlen(fname);
   strcpy(pathname,fname);
   
   if((token=strrchr(fname,CEPS)) == NULL){
      strcpy(pathname,".");
      return SUCCESS_EXEC;
   }
   
   int t=strlen(token);
        
   pathname[tt-t]='\0';
   
   return SUCCESS_EXEC;
}

// ******************************************************
// * GetFileTitle: return title without path but with extension
// * NB: see later how to get rid of extension
// ******************************************************

int GetFTitle(char *fname, char *titlename)
{
   char seps[] = SEPS;
   char *token, *runfname;
   runfname = new char [1024];
   strcpy(runfname,fname);
   strcpy(titlename,fname);
   
   token = strtok(runfname, seps);
   
   while( (token = strtok( NULL, seps )) != NULL ) strcpy(titlename,token);
   delete [] runfname;
   return SUCCESS_EXEC;
}

void StringToChar( char *c, string s )
{
   int len;
   len = s.length();
   s.copy( c, len, 0 );
   c[ len ] = 0;
}
