/*
 *  This file is part of the FREGENE package.
 *  Copyright (c) Clive J Hoggart   <c.hoggart@imperial.ac.uk>
 *                Marc Chadeau      <m.chadeau@imperial.ac.uk>
 *
 *  FREGENE is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License, version 2, as
 *  published by the Free Software Foundation.
 *
 *  FREGENE is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */ 
#include "rand.h"
#include "recomb2.h"

using namespace std;

Recomb2::Recomb2( uint32_t iChromoLength, double iOverallRate,
		  uint32_t HS_LENGTH, double iFractionInHS, double HS_spacing,
		  double iGC_RATE, uint32_t iGC_LENGTH, char *nameIN )
{
  ofstream RecombOutFile;

  string stringfilename1,stringfilename2;
  char charfilename[256];
  
  stringfilename1 = nameIN;
  stringfilename2 = stringfilename1 + "_RecombSummary" + ".txt";
  StringToChar( charfilename, stringfilename2 );
  RecombOutFile.open( charfilename );

  RecombOutFile << "Position Recomb_Rate" << endl;

   GC_RATE = iGC_RATE;
   GC_LENGTH = iGC_LENGTH;
   double a=4,b=3;
   OverallRate = iOverallRate;

   ChromoLength = iChromoLength;
   if( ChromoLength > 500000  )
      RegionLength = 500000;
   else
      RegionLength = ChromoLength;
   Regions = ChromoLength / RegionLength;
   
   SubsPerRegion = 10;
   SubLength = RegionLength / SubsPerRegion;
   FractionInHS = iFractionInHS;
   HotSpotLength = HS_LENGTH;

   double rate, sum=0.0;
   uint32_t posn=0;
   for( unsigned int i = 0; i < Regions; i++ ){
      rate = gengam( b, a );
      RegionRates.push_back( rate );
      sum += rate;
      RegionPosn.push_back(posn);
      posn += RegionLength;
   }

   uint32_t length;
   double subsum = 0, h, t = (double)HotSpotLength / SubLength;
   double intensity, sumintensity=0.0;
   SubRate.resize(Regions);
   SubPosn.resize(Regions);
   HS_posn.resize(Regions);
   HS_intensity.resize(Regions);
   for( unsigned int i = 0; i < Regions; i++ ){
      RegionRates[i] *= Regions*OverallRate/sum;
      subsum = 0;
      SubRate[i].resize( SubsPerRegion );
      HS_posn[i].resize( SubsPerRegion );
      HS_intensity[i].resize( SubsPerRegion );
      for( unsigned int j = 0; j < SubsPerRegion; j++ ){
         SubRate[i][j] = gengam( 0.3/RegionRates[i], 0.3 );
         subsum += SubRate[i][j];
      }
      posn = 0;
      for( unsigned int j = 0; j < SubsPerRegion; j++ ){
         sumintensity = 0.0;
         SubPosn[i].push_back(posn);
         posn += SubLength;
         SubRate[i][j] *= SubsPerRegion*RegionRates[i] / subsum;
         length = (uint32_t)gengam( 0.35/HS_spacing, 0.35 );
         while( length < SubLength ){
            HS_posn[i][j].push_back( length );
            intensity = gengam( 0.3/SubRate[i][j], 0.3 );
            HS_intensity[i][j].push_back( intensity );
            sumintensity += intensity;
            length += (uint32_t)gengam( 0.35/HS_spacing, 0.35 ) + HotSpotLength;
         }
         h = (SubRate[i][j]/sumintensity)*(FractionInHS - t)/(t*(1-t));
         for( unsigned int k = 0; k < HS_intensity[i][j].size(); k++ ){
            HS_intensity[i][j][k] *= h;
            HS_intensity[i][j][k] *= HotSpotLength;
         }
         HS_intensity[i][j].push_back( SubLength*SubRate[i][j]*(1-FractionInHS)/(1-t) );
      }
   }

//    RecombOutFile << "Region rates" << endl;
//    for( unsigned int i = 0; i < Regions; i++ )
//       RecombOutFile << RegionRates[i] << " ";
//    RecombOutFile << endl;

//    RecombOutFile << "Sub rates" << endl;
//    for( unsigned int i = 0; i < Regions; i++ ){
//       for( unsigned int j = 0; j < SubsPerRegion; j++ )
//          RecombOutFile << SubRate[i][j] << " ";
//       RecombOutFile << endl;
//    }

   for( unsigned int i = 0; i < Regions; i++ ){
      for( unsigned int j = 0; j < SubsPerRegion; j++ ){
//          RecombOutFile << i << " " << j  << " -1 ";
         RecombOutFile << RegionPosn[i]+SubPosn[i][j] << " "
              << HS_intensity[i][j][HS_intensity[i][j].size() - 1]/SubLength << endl;
         for( unsigned int k = 0; k < HS_intensity[i][j].size() - 1; k++ ){
//             RecombOutFile << i << " " << j  << " " << k << " ";
            RecombOutFile << RegionPosn[i]+SubPosn[i][j]+HS_posn[i][j][k] << " "
                 << HS_intensity[i][j][HS_intensity[i][j].size() - 1]/SubLength
               + HS_intensity[i][j][k]/HotSpotLength << endl;
//             RecombOutFile << i << " " << j  << " " << k << " ";
            RecombOutFile << RegionPosn[i]+SubPosn[i][j]+HS_posn[i][j][k]+HotSpotLength << " "
                 << HS_intensity[i][j][HS_intensity[i][j].size() - 1]/SubLength << endl;
         }
      }
   }
   RecombOutFile.close();
}

void Recomb2::UpdateRecombinationRate( double factor )
{
   OverallRate *= factor;
   GC_RATE *= factor;
}
double Recomb2::GetRecombinationRate()
{
   return OverallRate;
}

void Recomb2::GetRecombinations(vector<uint32_t>& BreakPointVector)
{
   uint32_t region=0, sub=0, hs=0, posn;

// Generate cross-overs
   unsigned short recoms = genpoi( (double)OverallRate*ChromoLength );
   unsigned short gc = genpoi( (double)GC_RATE*ChromoLength );
   if( recoms + gc != 0 ){
      BreakPointVector.push_back(0);
      BreakPointVector.push_back(ChromoLength);
   }
   for( unsigned int ii = 0; ii < recoms; ii++ ){
      region = SampleFromDiscrete( RegionRates );
      sub = SampleFromDiscrete( SubRate[region] );
      hs = SampleFromDiscrete( HS_intensity[region][sub] );
      if( hs == HS_intensity[region][sub].size() - 1 ){
         posn = SampleFromRange( 0, SubLength ) + SubPosn[region][sub] + RegionPosn[region];
         BreakPointVector.push_back( posn );
      }
      else{
         posn = SampleFromRange( 0, HotSpotLength ) + HS_posn[region][sub][hs] + SubPosn[region][sub] + RegionPosn[region];
//          if( posn > ChromoLength )
//             cout << "bollox\n";
         if( posn < ChromoLength )
            BreakPointVector.push_back( posn );
      }
      double sum=0;
      unsigned int j;
      for( j=0; j < HS_intensity[region][sub].size()-1; j++ )
         sum += HS_intensity[region][sub][j];
//      cout << sum / (sum + HS_intensity[region][sub][j]) << endl;
//      cout << sum / (SubRate[region][sub]*SubLength) << endl;
   }

// Generate gene conversions
   for( unsigned int i = 0; i < gc; i++ ){
      posn = SampleFromRange(1,ChromoLength-GC_LENGTH);
      if( posn < ChromoLength )
         BreakPointVector.push_back( posn );
      if( posn + GC_LENGTH < ChromoLength )
         BreakPointVector.push_back( (posn + GC_LENGTH) );
//       if( posn > ChromoLength )
//          cout << "bollox\n";
   }
   if( recoms + gc != 0 )
      sort( BreakPointVector.begin(), BreakPointVector.end() );
}

