/*
 *  This file is part of the FREGENE package.
 *  Copyright (c) Clive J Hoggart   <c.hoggart@imperial.ac.uk>
 *                Taane G Clark     <tgc@well.ox.ac.uk>
 *                Marc Chadeau      <m.chadeau@imperial.ac.uk>
 *
 *  FREGENE is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License, version 2, as
 *  published by the Free Software Foundation.
 *
 *  FREGENE is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */ 

#include <cstdlib>
#include <cstring>
#include <ctime>
#include <cmath>
#include "xml_file_write.h"

//========================================================================>>>
//=========================MaXmlTagWrite==================================

MaXmlTagWrite::MaXmlTagWrite()
{
	XF=NULL;
}

MaXmlTagWrite::MaXmlTagWrite(FILE *f, char version[10])
{
	SetXmlFile(f);
	WriteDeclaration(version);
}

MaXmlTagWrite::MaXmlTagWrite(char *s, char version[10])
{
	XF=OpenXmlFile(s);
	WriteDeclaration(version);
}

MaXmlTagWrite::~MaXmlTagWrite()
{
	if(XF) fclose(XF);
	XF=NULL;
}



FILE* MaXmlTagWrite::OpenXmlFile(char *s)
{
	FILE *fx=NULL;

	fx=fopen(s,"wb");

	return fx;
}

int MaXmlTagWrite::CloseXmlFile(FILE *f)
{
	if(f) 
		if(fclose(f)) return MA_CLOSE_FILE_ERROR;
		else 
		{
			f=NULL;
			return MA_NO_ERROR;
		}
   else return MA_NO_ACTION;
}


void MaXmlTagWrite::SetXmlFile(FILE *f) 
{
	if(f) XF=f;
	else throw MaXmlFileException(MA_XML_FILE_ERROR);
}


int MaXmlTagWrite::WriteDeclaration(const char version[10])
{	
	if(!XF) return MA_XML_FILE_ERROR;

	char *command=new char[1024];
	sprintf(command,"<?xml version='%s' ?>\n",version);
	int n=fwrite(command,1,strlen(command),XF);

	delete [] command;
	return n;
}



int MaXmlTagWrite::WriteDocType()  //to perform
{	
	if(!XF) return MA_XML_FILE_ERROR;

	char *command=new char[1024];
	sprintf(command,"\n");
	int n=fwrite(command,1,strlen(command),XF);

	delete [] command;
	return n;
}


int MaXmlTagWrite::WriteComment(const char *s)  
{	
	if(!XF) return MA_XML_FILE_ERROR;

	char *command=new char[1024];
	sprintf(command,"<!-- %s -->",s);
	int n=fwrite(command,1,strlen(command),XF);

	delete [] command;

	return n;
}

int MaXmlTagWrite::WriteString(const char *s)  
{	
	if(!XF) return MA_XML_FILE_ERROR;
	int tn =strlen(s);
	char *command=new char[tn+1];
	sprintf(command,"%s",s);
	int n=fwrite(command,1,strlen(command),XF);

	delete [] command;

	return n;
}

int MaXmlTagWrite::WriteSpace()  
{	
	if(!XF) return MA_XML_FILE_ERROR;

	int n=fwrite("\n",1,strlen("\n"),XF);

	return n;
}


int MaXmlTagWrite::WriteLine(const char *s)  
{	
	if(!XF) return MA_XML_FILE_ERROR;

	char *command=new char[1024];
	sprintf(command,"%s",s);
	int n=fwrite(command,1,strlen(command),XF);

	delete [] command;

	return n;
}


int MaXmlTagWrite::WriteTab()  
{	
	if(!XF) return MA_XML_FILE_ERROR;

	int n=fwrite("\t",1,strlen("\t"),XF);

	return n;
}


int MaXmlTagWrite::WriteEmptyElement(char *s)
{	
	if(!XF) return MA_XML_FILE_ERROR;

	char *command=new char[1024];
	sprintf(command,"<%s/>\n",s);
	int n=fwrite(command,1,strlen(command),XF);

	delete [] command;
	return n;
}

int MaXmlTagWrite::WriteStat(STATS stat)
{
	char *command=new char[1024];
	

	StartTag("RANGE");
	sprintf(command,"%ld %ld",stat.min_max.start,stat.min_max.end);
	int n=fwrite(command,1,strlen(command),XF);
	EndTag("RANGE");

	StartTag("SIZE");
	sprintf(command,"%d",stat.size);
	n+=fwrite(command,1,strlen(command),XF);
	EndTag("SIZE");

	StartTag("MEAN");
	sprintf(command,"%.2lf",stat.mean);
	n+=fwrite(command,1,strlen(command),XF);
	EndTag("MEAN");

	StartTag("SIGMA");
	sprintf(command,"%.2lf",stat.sigma);
	n+=fwrite(command,1,strlen(command),XF);
	EndTag("SIGMA");

	StartTag("S_BY_M");
	sprintf(command,"%.4lf",stat.sigma/stat.mean);
	n+=fwrite(command,1,strlen(command),XF);
	EndTag("S_BY_M");

	delete [] command;

	return n;
}

int MaXmlTagWrite::WriteSaturation(unsigned long ns, double mean, double proportion)
{
	char *command=new char[1024];
	
	StartTag("SATURATION");

	StartTag("NBSAT");
	sprintf(command,"%lu ",ns);
	int n=fwrite(command,1,strlen(command),XF);
	EndTag("NBSAT");

	StartTag("MEANSAT");
	sprintf(command,"%.2lf",mean);
	n+=fwrite(command,1,strlen(command),XF);
	EndTag("MEANSAT");

	StartTag("PROPSAT");
	sprintf(command,"%.2lf",proportion);
	n+=fwrite(command,1,strlen(command),XF);
	EndTag("PROPSAT");

	EndTag("SATURATION");

	delete [] command;

	return n;
}



int MaXmlTagWrite::StartTag(const char *s)
{	
	if(!XF) return MA_XML_FILE_ERROR;

	char *command=new char[1024];
	sprintf(command,"<%s>",s);
	int n=fwrite(command,1,strlen(command),XF);

	delete [] command;
	return n;
}


int MaXmlTagWrite::EndTag(const char *s)
{	
	if(!XF) return MA_XML_FILE_ERROR;

	char *command=new char[1024];
	sprintf(command,"</%s>\n",s);
	int n=fwrite(command,1,strlen(command),XF);

	delete [] command;
	return n;
}


int MaXmlTagWrite::StartCDATA()
{	
	if(!XF) return MA_XML_FILE_ERROR;

	char *command=new char[1024];
	sprintf(command,"<![CDATA[\n");
	int n=fwrite(command,1,strlen(command),XF);

	delete [] command;
	return n;
}

int MaXmlTagWrite::EndCDATA()
{	
	if(!XF) return MA_XML_FILE_ERROR;

	char *command=new char[1024];
	sprintf(command,"\n]]>\n");
	int n=fwrite(command,1,strlen(command),XF);

	delete [] command;
	return n;
}

int MaXmlTagWrite::StartHead()  
{	
	if(!XF) return MA_XML_FILE_ERROR;

	StartTag("HEAD");

	return MA_NO_ERROR;
}

int MaXmlTagWrite::EndHead()  
{	
	if(!XF) return MA_XML_FILE_ERROR;

	EndTag("HEAD");

	return MA_NO_ERROR;
}


int MaXmlTagWrite::StartBinData()  
{	
	if(!XF) return MA_XML_FILE_ERROR;

	StartTag("DATA");
	StartCDATA();

	return MA_NO_ERROR;
}

int MaXmlTagWrite::EndBinData()  
{	
	if(!XF) return MA_XML_FILE_ERROR;
	
	EndCDATA();
	EndTag("DATA");

	return MA_NO_ERROR;
}

int MaXmlTagWrite::StartAData()  
{	
	if(!XF) return MA_XML_FILE_ERROR;

	StartTag("DATA");

	return MA_NO_ERROR;
}

int MaXmlTagWrite::EndAData()  
{	
	if(!XF) return MA_XML_FILE_ERROR;
	
	EndTag("DATA");

	return MA_NO_ERROR;
}



int MaXmlTagWrite::WriteAuthor(char *s)  
{	
	if(!XF) return MA_XML_FILE_ERROR;

	StartTag("AUTHOR");
	int n=fwrite(s,1,strlen(s),XF);
	EndTag("AUTHOR");

	return n;
}

int MaXmlTagWrite::WriteDate()  
{	
	if(!XF) return MA_XML_FILE_ERROR;

	char *command=new char[1024];
	StartTag("DATE");
   time_t ltime;
   time( &ltime );
   strcpy(command,ctime( &ltime ));
	int n=fwrite(command,1,strlen(command),XF);
	EndTag("DATE");

	delete [] command;
	return n;
}

int MaXmlTagWrite::WriteType(char *s)  
{	
	if(!XF) return MA_XML_FILE_ERROR;

	char *command=new char[1024];
	StartTag("TYPE");
	if((strcmp("T",s)!=0) || (strcmp("B",s)!=0)) strcpy(command,"UT");
	else strcpy(command,s);
	int n=fwrite(command,1,strlen(command),XF);
	EndTag("TYPE");

	delete [] command;
	return n;
}

//========================================================================
//========================================================================>>>

//========================================================================>>>
//=========================MaXml_CS_TagWrite==================================

MaXml_CS_TagWrite::MaXml_CS_TagWrite(FILE *f, char version[10])
						:MaXmlTagWrite(f,version)
{
	WriteComment("This a CS (Colonies Segmentation) File");	
	WriteSpace();
}

MaXml_CS_TagWrite::MaXml_CS_TagWrite(char *s, char version[10])
						:MaXmlTagWrite(s,version)
{
	WriteComment("This a CS (Colonies Segmentation) File");
	WriteSpace();
}

int MaXml_CS_TagWrite::WriteImageName(char *s)  
{	
	if(!XF) return MA_XML_FILE_ERROR;

	StartTag("IMAGE");	
	int n=fwrite(s,1,strlen(s),XF);
	EndTag("IMAGE");

	return n;
}


int MaXml_CS_TagWrite::WriteSize(int dimx, int dimy)  
{	
	if(!XF) return MA_XML_FILE_ERROR;

	char *command=new char[1024];
	
	StartTag("SIZE");
	sprintf(command,"%d %d",dimx,dimy);
	int n=fwrite(command,1,strlen(command),XF);
	EndTag("SIZE");

	delete [] command;
	return n;
}

int MaXml_CS_TagWrite::WriteNbPeaks(int np)
{
	if(!XF) return MA_XML_FILE_ERROR;

	char *command=new char[1024];
	
	StartTag("NBPEAKS");
	sprintf(command,"%d",np);
	int n=fwrite(command,1,strlen(command),XF);
	EndTag("NBPEAKS");

	delete [] command;
	return n;
}

int MaXml_CS_TagWrite::WriteMinbp(int np) //Min nb pixel
{
	if(!XF) return MA_XML_FILE_ERROR;

	char *command=new char[1024];
	
	StartTag("MIN_NB_PIX");
	sprintf(command,"%d",np);
	int n=fwrite(command,1,strlen(command),XF);
	EndTag("MIN_NB_PIX");

	delete [] command;
	return n;
}


int MaXml_CS_TagWrite::WriteMaxbp(int np) //Max nb pixel
{
	if(!XF) return MA_XML_FILE_ERROR;

	char *command=new char[1024];
	
	StartTag("MAX_NB_PIX");
	sprintf(command,"%d",np);
	int n=fwrite(command,1,strlen(command),XF);
	EndTag("MAX_NB_PIX");

	delete [] command;
	return n;
}


int MaXml_CS_TagWrite::WriteMinPeakSignal(int np) //Min Peak Signal
{
	if(!XF) return MA_XML_FILE_ERROR;

	char *command=new char[1024];
	
	StartTag("MIN_PEAK_SIGNAL");
	sprintf(command,"%d",np);
	int n=fwrite(command,1,strlen(command),XF);
	EndTag("MIN_PEAK_SIGNAL");

	delete [] command;
	return n;
}


int MaXml_CS_TagWrite::WriteBaseParam(int np) //baseline parameter
{
	if(!XF) return MA_XML_FILE_ERROR;

	char *command=new char[1024];
	
	StartTag("BASE_PARAM");
	sprintf(command,"%d",np);
	int n=fwrite(command,1,strlen(command),XF);
	EndTag("BASE_PARAM");

	delete [] command;
	return n;
}

int MaXml_CS_TagWrite::WriteStopParam(int np) //stop parameter
{
	if(!XF) return MA_XML_FILE_ERROR;

	char *command=new char[1024];
	
	StartTag("STOP_PARAM");
	sprintf(command,"%d",np);
	int n=fwrite(command,1,strlen(command),XF);
	EndTag("STOP_PARAM");

	delete [] command;
	return n;
}

int MaXml_CS_TagWrite::WritePcr(double np) // % Pcr
{
	if(!XF) return MA_XML_FILE_ERROR;

	char *command=new char[1024];
	
	StartTag("PCR");
	sprintf(command,"%.2lf",np);
	int n=fwrite(command,1,strlen(command),XF);
	EndTag("PCR");

	delete [] command;
	return n;
}


int MaXml_CS_TagWrite::WriteUseFilter(int ufilter)  
{
	if(!XF) return MA_XML_FILE_ERROR;

	char *command=new char[1024];
	
	StartTag("FILTER");
	sprintf(command,"%d",(int)ufilter);
	int n=fwrite(command,1,strlen(command),XF);
	EndTag("FILTER");

	delete [] command;
	return n;
}


int MaXml_CS_TagWrite::WriteFilter(int filter) 
{
	if(!XF) return MA_XML_FILE_ERROR;

	char *command=new char[1024];
	
	StartTag("FILTER_USED");
	sprintf(command,"%d",(int)filter);
	int n=fwrite(command,1,strlen(command),XF);
	EndTag("FILTER_USED");

	delete [] command;
	return n;
}


int MaXml_CS_TagWrite::WriteFilterPar1(int fpar)  
{
	if(!XF) return MA_XML_FILE_ERROR;

	char *command=new char[1024];
	
	StartTag("FILTER_SIZE");
	sprintf(command,"%d",(int)fpar);
	int n=fwrite(command,1,strlen(command),XF);
	EndTag("FILTER_SIZE");

	delete [] command;
	return n;
}


int MaXml_CS_TagWrite::WriteFilterPar2(double fpar) 
{
	if(!XF) return MA_XML_FILE_ERROR;

	char *command=new char[1024];
	
	StartTag("FILTER_SIZE");
	sprintf(command,"%.2lf",(double)fpar);
	int n=fwrite(command,1,strlen(command),XF);
	EndTag("FILTER_SIZE");

	delete [] command;
	return n;
}



int MaXml_CS_TagWrite::WriteRange(RANGE xr, RANGE yr) //range
{
	if(!XF) return MA_XML_FILE_ERROR;

	char *command=new char[1024];
	
	StartTag("RANGE_ZONE");
	sprintf(command,"%ld %ld %ld %ld",xr.start,xr.end,yr.start,yr.end);
	int n=fwrite(command,1,strlen(command),XF);
	EndTag("RANGE_ZONE");

	delete [] command;
	return n;
}

int MaXml_CS_TagWrite::WriteDynamic(RANGE d) 
{
	if(!XF) return MA_XML_FILE_ERROR;

	char *command=new char[1024];
	
	StartTag("DYNAMIC");
	sprintf(command,"%ld %ld",d.start,d.end);
	int n=fwrite(command,1,strlen(command),XF);
	EndTag("DYNAMIC");

	delete [] command;
	return n;
}

int MaXml_CS_TagWrite::WriteFluo(bool f) 
{
	if(!XF) return MA_XML_FILE_ERROR;

	char *command=new char[1024];
	
	StartTag("FLUO");
	sprintf(command,"%d",(int)f);
	int n=fwrite(command,1,strlen(command),XF);
	EndTag("FLUO");

	delete [] command;
	return n;
}

int MaXml_CS_TagWrite::WriteHalo(bool h) 
{
	if(!XF) return MA_XML_FILE_ERROR;

	char *command=new char[1024];
	
	StartTag("HALOGEN");
	sprintf(command,"%d",(int)h);
	int n=fwrite(command,1,strlen(command),XF);
	EndTag("HALOGEN");

	delete [] command;
	return n;
}

int MaXml_CS_TagWrite::WriteExposureTime(double et) 
{
	if(!XF) return MA_XML_FILE_ERROR;

	char *command=new char[1024];
	
	StartTag("EXPOSURE_TIME");
	sprintf(command,"%.2lf",et);
	int n=fwrite(command,1,strlen(command),XF);
	EndTag("EXPOSURE_TIME");

	delete [] command;
	return n;
}


int MaXml_CS_TagWrite::WriteLightDose(double ld) 
{
	if(!XF) return MA_XML_FILE_ERROR;

	char *command=new char[1024];
	
	StartTag("LIGHT_DOSE");
	sprintf(command,"%.2lf",ld);
	int n=fwrite(command,1,strlen(command),XF);
	EndTag("LIGHT_DOSE");

	delete [] command;
	return n;
}


int MaXml_CS_TagWrite::WriteCSHead(PEAK_PICKING_PARAM pp)
{
	///Obligatory Tag for CS XML File
	//image name, image size, peak number

	StartTag("HEAD");
	WriteDate();
	WriteImageName(pp.image);
	if(strcmp(pp.author,"")!=0) WriteAuthor(pp.author);
	WriteSize(pp.dimx,pp.dimy);
	WriteRange(pp.xr, pp.yr); 
	WriteMinbp(pp.min_nb_pix);					
	WriteMaxbp(pp.max_nb_pix);				   
	WriteMinPeakSignal(pp.min_peak_signal);	   
	WriteBaseParam(pp.baseline_parameter);		   
	WriteStopParam(pp.stop_parameter);		   
	WritePcr(pp.pcr);					
	WriteUseFilter(pp.use_filter);				
	WriteFilter(pp.filter);				
	WriteFilterPar1(pp.fpar1);				
	WriteFilterPar2(pp.fpar2);				
	WriteNbPeaks(pp.nb_peaks);
	WriteDynamic(pp.dynamic);
	WriteFluo(pp.fluo);
	WriteHalo(pp.halogen);
	WriteExposureTime(pp.ExposureTime);
	WriteLightDose(pp.LightIntensity);

	EndTag("HEAD");

	return MA_NO_ERROR;
}


//========================================================================
//========================================================================>>>

//========================================================================
//=====================PeakMatching Tag write============================>>>

//========================================================================>>>
//=========================MaXml_PM_TagWrite==================================

MaXml_PM_TagWrite::MaXml_PM_TagWrite(FILE *f, char version[10])
						:MaXmlTagWrite(f,version)
{
	WriteComment("This a PM (Peaks Matching) File");	
}

MaXml_PM_TagWrite::MaXml_PM_TagWrite(char *s, char version[10])
						:MaXmlTagWrite(s,version)
{
	WriteComment("This a PM (Peaks Matching) File");	
}

int MaXml_PM_TagWrite::WriteNbFilesTreated(unsigned short np)
{
	if(!XF) return MA_XML_FILE_ERROR;

	char *command=new char[1024];
	
	StartTag("NB_FILE_TREATED");
	sprintf(command,"%d",np);
	int n=fwrite(command,1,strlen(command),XF);
	EndTag("NB_FILE_TREATED");

	delete [] command;
	return n;
}

int MaXml_PM_TagWrite::WriteRefFile(char *s, unsigned char np)  
{	
	if(!XF) return MA_XML_FILE_ERROR;

	char *command=new char[1024];
	
	StartTag("REF_FILE");
	sprintf(command,"%d %s",np,s);
	int n=fwrite(command,1,strlen(command),XF);
	EndTag("REF_FILE");

	delete [] command;
	return n;
}

int MaXml_PM_TagWrite::WriteTreatedFile(char *s)  
{	
	if(!XF) return MA_XML_FILE_ERROR;

	char *command=new char[1024];
	
	StartTag("TREATED_FILE");
	sprintf(command,"%s",s);
	int n=fwrite(command,1,strlen(command),XF);
	EndTag("TREATED_FILE");

	delete [] command;
	return n;
}

int MaXml_PM_TagWrite::WriteNumColumn(unsigned char np)
{
	if(!XF) return MA_XML_FILE_ERROR;

	char *command=new char[1024];
	
	StartTag("NUM_COLUMN");
	sprintf(command,"%d",np);
	int n=fwrite(command,1,strlen(command),XF);
	EndTag("NUM_COLUMN");

	delete [] command;
	return n;
}

int MaXml_PM_TagWrite::WriteContinuity(unsigned short np)
{
	if(!XF) return MA_XML_FILE_ERROR;

	char *command=new char[1024];
	
	StartTag("CONTINUITY");
	sprintf(command,"%d",np);
	int n=fwrite(command,1,strlen(command),XF);
	EndTag("CONTINUITY");

	delete [] command;
	return n;
}


int MaXml_PM_TagWrite::WriteMergeDist(unsigned char np)
{
	if(!XF) return MA_XML_FILE_ERROR;

	char *command=new char[1024];
	
	StartTag("MERGE_DIST");
	sprintf(command,"%d",np);
	int n=fwrite(command,1,strlen(command),XF);
	EndTag("MERGE_DIST");

	delete [] command;
	return n;
}



int MaXml_PM_TagWrite::WriteMaxTranslation(unsigned short np)
{
	if(!XF) return MA_XML_FILE_ERROR;

	char *command=new char[1024];
	
	StartTag("MAX_TRANSLATION");
	sprintf(command,"%d",np);
	int n=fwrite(command,1,strlen(command),XF);
	EndTag("MAX_TRANSLATION");

	delete [] command;
	return n;
}


int MaXml_PM_TagWrite::WriteCrims(unsigned short np)
{
	if(!XF) return MA_XML_FILE_ERROR;

	char *command=new char[1024];
	
	StartTag("CRIMS");
	sprintf(command,"%d",np);
	int n=fwrite(command,1,strlen(command),XF);
	EndTag("CRIMS");

	delete [] command;
	return n;
}


int MaXml_PM_TagWrite::WritePmParams(PEAK_MATCHING_PARAM pm)
{
	StartTag("PM_PARAMS");
	WriteDate();
	WriteRefFile(pm.ref_file_name, pm.ref_file);
	WriteContinuity(pm.continuity);
	WriteNumColumn(pm.num_col);
	WriteMergeDist(pm.merge_dist);
	WriteMaxTranslation(pm.max_translation);
	WriteCrims(pm.crims);
	WriteNbFilesTreated(pm.nb_file_treated);
	EndTag("PM_PARAMS");
	
	return MA_NO_ERROR;
}

//========================================================================>>>
//=========================MaXml_SQ_TagWrite==================================

MaXml_SQ_TagWrite::MaXml_SQ_TagWrite(FILE *f, char version[10])
						:MaXmlTagWrite(f,version)
{

}

int MaXml_SQ_TagWrite::WriteLine(char *s)  
{	
	if(!XF) return MA_XML_FILE_ERROR;

	int tn =strlen(s);
	char *command=new char[tn+1];
	sprintf(command,"%s",s);
	int n=fwrite(command,1,strlen(command),XF);

	delete [] command;

	return n;
}


int MaXml_SQ_TagWrite::WriteFormat()
{	
	if(!XF) return MA_XML_FILE_ERROR;

	char *command=new char[1024];
	sprintf(command,"%s", "<?xml-stylesheet href=\"/experiment/experiment.xsl\" type=\"text/xsl\"?>\n");
	int n=fwrite(command,1,strlen(command),XF);

	delete [] command;
	return n;
}

int MaXml_SQ_TagWrite::WriteFormat4x4()
{	
	if(!XF) return MA_XML_FILE_ERROR;

	char *command=new char[1024];
	sprintf(command,"%s", "<?xml-stylesheet href=\"/experiment/4x4.xsl\" type=\"text/xsl\"?>\n");
	int n=fwrite(command,1,strlen(command),XF);

	delete [] command;
	return n;
}

int MaXml_SQ_TagWrite::WriteFormatSLV()
{	
	if(!XF) return MA_XML_FILE_ERROR;

	char *command=new char[1024];
	sprintf(command,"%s", "<?xml-stylesheet href=\"/experiment/slv.xsl\" type=\"text/xsl\"?>\n");
	int n=fwrite(command,1,strlen(command),XF);

	delete [] command;
	return n;
}

int MaXml_SQ_TagWrite::WriteExt(int ord, char t, float brut, float nor, float std)
{
unsigned n;
int flag_error = 0;

	if(!XF) return MA_XML_FILE_ERROR;
	char *command=new char[1024];
	StartTag("EXT");
	WriteSpace();
	StartTag("ORD");
	sprintf(command,"%d", ord);
	if((n=fwrite(command,1,strlen(command),XF))<strlen(command)) flag_error = 1;
	EndTag("ORD");
	StartTag("TYPE");
	sprintf(command,"%c", t);
	if((n=fwrite(command,1,strlen(command),XF))<strlen(command)) flag_error = 1;
	EndTag("TYPE");
	StartTag("BRUT");
	sprintf(command,"%.2f", brut);
	if((n=fwrite(command,1,strlen(command),XF))<strlen(command)) flag_error = 1;
	EndTag("BRUT");
	StartTag("NORMALIZED");
	sprintf(command,"%.2f", nor);
	if((n=fwrite(command,1,strlen(command),XF))<strlen(command)) flag_error = 1;
	EndTag("NORMALIZED");
	StartTag("STD");
	sprintf(command,"%.2f", std);
	if((n=fwrite(command,1,strlen(command),XF))<strlen(command)) flag_error = 1;
	EndTag("STD");
	EndTag("EXT");

	delete [] command;
	return flag_error;


}

int MaXml_SQ_TagWrite::WriteMot(char mot[31], int scr, int dist, float con)
{
unsigned n;
int flag_error = 0;

	if(!XF) return MA_XML_FILE_ERROR;
	char *command=new char[1024];
	StartTag("MOT");
	WriteSpace();
	StartTag("SEQ");
	sprintf(command,"%s", mot);
	if((n=fwrite(command,1,strlen(command),XF))<strlen(command)) flag_error = 1;
	EndTag("SEQ");
	StartTag("SCORE");
	sprintf(command,"%d", scr);
	if((n=fwrite(command,1,strlen(command),XF))<strlen(command)) flag_error = 1;
	EndTag("SCORE");
	StartTag("DISTANCE");
	sprintf(command,"%d", dist);
	if((n=fwrite(command,1,strlen(command),XF))<strlen(command)) flag_error = 1;
	EndTag("DISTANCE");
	StartTag("CONFIDENCE");
	sprintf(command,"%.2f", con);
	if((n=fwrite(command,1,strlen(command),XF))<strlen(command)) flag_error = 1;
	EndTag("CONFIDENCE");
	EndTag("MOT");

	delete [] command;
	return flag_error;


}

