/*
 *  This file is part of the FREGENE package.
 *  Copyright (c) Clive J Hoggart   <c.hoggart@imperial.ac.uk>
 *                Taane G Clark     <tgc@well.ox.ac.uk>
 *                Marc Chadeau      <m.chadeau@imperial.ac.uk>
 *
 *  FREGENE is free software; you can redistribute it and/or modify it
 *  under the terms of the GNU General Public License, version 2, as
 *  published by the Free Software Foundation.
 *
 *  FREGENE is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */ 
#ifndef XML_FILE_WRITE_H

#define XML_FILE_WRITE_H

#include <cstdio>
#include "maerrno.h"
#include "maexcepcls.h"
#include "ma_struct.h"



class MaXmlTagWrite
{
protected :
	FILE *XF;

public :
	//MaXmlTagWrite();
	MaXmlTagWrite();
	MaXmlTagWrite(FILE *f, char version[10]="1.0");
	MaXmlTagWrite(char *s, char version[10]="1.0");

	~MaXmlTagWrite();

	FILE *GetXmlFile() {return XF;}
	void SetXmlFile(FILE *f); 
	FILE *OpenXmlFile(char *s);
	int  CloseXmlFile(FILE *f);
	//writing xml declaration
	int WriteDeclaration(const char version[10]="1.0");
	

	//writing xml DocType
	int WriteDocType();

	
	//writing xml comment
	int WriteComment(const char *s);

	//writing string : redefinition of fwrite
	int WriteString(const char *s);


	//writing Space
	int WriteSpace();

	//writing Tabulation
	int WriteTab();

	//writing line
	int WriteLine(const char *s);

	//writing Empty Element
	int WriteEmptyElement(char *s);


	//writing begin tag
	int StartTag(const char *s);
	//wrinting end tag
	int EndTag(const char *s);
	
	//writing begin tag CDATA
	int StartCDATA();
	//wrinting end tag CDATA
	int EndCDATA();

	//writing begin tag head
	int StartHead();
	//wrinting end tag head
	int EndHead();

	//writing begin tag manip name
	int StartManip();
	//wrinting end tag manip name
	int EndManip();


	//writing begin tag Data in case of binary
	int StartBinData();
	//wrinting end tag Data in case of binary
	int EndBinData();

	//writing begin tag Data in case of ascii
	int StartAData();
	//wrinting end tag Data in case of ascii
	int EndAData();


	//write fixed tag
	int WriteAuthor(char *s);
	int WriteDate();
	int WriteDataFormat();           //Binaries Data or ascii data
	int WriteType(char *c);			   //Type de fichier
	int WriteDescription(char *des); //Description de ce que contient le champs DATA
	int WriteStat(STATS stat);			//write diffrent statistic compeonent
	int WriteSaturation(unsigned long n, double mean, double proportion); //write saturation parameter
};

//========================================================================
//========================================================================>>>

//========================================================================>>>
//=========================MaXml_CS_TagWrite==================================

class MaXml_CS_TagWrite : public  MaXmlTagWrite
{

public :
	MaXml_CS_TagWrite(FILE *f, char version[10]="1.0");
	MaXml_CS_TagWrite(char *s, char version[10]="1.0");

	int WriteImageName(char *s);			//Image File
	int WriteSize(int dimx, int dimy);  //ImageSize
	int WriteNbPeaks(int n);				//Number of Peaks detected
	int WriteMinbp(int np);					//Min nb pixel
	int WriteMaxbp(int np);				   //Max nb pixel
	int WriteMinPeakSignal(int np);	   //Min Peak Signal
	int WriteBaseParam(int np);		   //baseline parameter
	int WriteStopParam(int np);		   //stop parameter
	int WritePcr(double np);					// % Pcr
	int WriteUseFilter(int ufilter);			
	int WriteFilter(int filter);			
	int WriteFilterPar1(int fpar);			
	int WriteFilterPar2(double fpar);			
	int WriteRange(RANGE xr, RANGE yr); //range
	int WriteDynamic(RANGE d);				//dynamique de l'image
	int WriteFluo(bool f);					//type de la source de lumiere
	int WriteHalo(bool h);					//
	int WriteExposureTime(double et);	//temps d'exposition
	int WriteLightDose(double ld);		//quantit de lumiere	


	int WriteCSHead(PEAK_PICKING_PARAM pp);						//Specific Head

};



//========================================================================
//========================================================================>>>

//========================================================================>>>
//=========================MaXml_PM_TagWrite==================================

class MaXml_PM_TagWrite : public  MaXmlTagWrite
{

public :
	MaXml_PM_TagWrite(FILE *f, char version[10]="1.0");
	MaXml_PM_TagWrite(char *s, char version[10]="1.0");

	int WriteNbFilesTreated(unsigned short np);
	int WriteRefFile(char *s, unsigned char np);
	int WriteTreatedFile(char *s);  
	int WriteNumColumn(unsigned char np);
	int WriteContinuity(unsigned short np);
	int WriteMergeDist(unsigned char np);
	int WriteMaxTranslation(unsigned short np);
	int WriteCrims(unsigned short np);
	int WritePmParams(PEAK_MATCHING_PARAM pm);
};

//=========================MaXml_SQ_TagWrite==================================

class MaXml_SQ_TagWrite : public  MaXmlTagWrite
{
public :

	MaXml_SQ_TagWrite(FILE *f, char version[10]="1.0");
	//writing command line for xml formating file
	int WriteFormat();

	//writing command line for 4x4 xml formating file
	int WriteFormat4x4();

	//writing command line for slv xml formating file (cleavable linker)
       	int WriteFormatSLV();

	// writing line
	int WriteLine(char *s);

	// writing delta for an extension
	int WriteExt(int ord, char t, float brut, float nor, float std);

	// writing motive
	int WriteMot(char mot[31], int scr, int dist, float con);


};
#endif 

